/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.connector.commands;

import static com.openexchange.usm.connector.commands.CommandConstants.FOLDERID;
import static com.openexchange.usm.connector.commands.CommandConstants.OBJECTIDS;
import static com.openexchange.usm.connector.commands.CommandConstants.PROPERTIES;
import static com.openexchange.usm.connector.commands.CommandConstants.SESSIONID;
import static com.openexchange.usm.connector.commands.CommandConstants.SYNCID;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.usm.api.datatypes.DataType;
import com.openexchange.usm.api.exceptions.USMException;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.api.session.Folder;
import com.openexchange.usm.connector.exceptions.DataObjectNotFoundException;
import com.openexchange.usm.connector.exceptions.InvalidUUIDException;
import com.openexchange.usm.connector.exceptions.MultipleOperationsOnDataObjectException;
import com.openexchange.usm.json.ConnectorBundleErrorCodes;
import com.openexchange.usm.json.USMJSONAPIException;
import com.openexchange.usm.json.USMJSONServlet;
import com.openexchange.usm.json.response.ResponseObject;
import com.openexchange.usm.json.response.ResponseStatusCode;
import com.openexchange.usm.session.dataobject.DataObjectSet;

/**
 * Handler for the ReadData USM-JSON-Command.
 * @author ldo
 *
 */
public class ReadDataHandler extends NormalCommandHandler {
	private static final String UUID = "uuid";
	private static final String[] REQUIRED_PARAMETERS = { SESSIONID, SYNCID, FOLDERID, OBJECTIDS, PROPERTIES };
	private static final String[] OPTIONAL_PARAMETERS = {};

    public ReadDataHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
        super(servlet, request);
    }

	@Override
	public ResponseObject handleRequest() throws USMJSONAPIException {
		String folderID = getStringParameter(FOLDERID, null);
		long syncId = getSyncID();
		JSONArray objectids = getJSONArray(_parameters, OBJECTIDS);
		JSONArray properties = getJSONArray(_parameters, PROPERTIES);
		List<String> fields = createFieldsListFromParameter(properties);
		DataObject[] resultElements = readDataElements(syncId, folderID, objectids, fields);
		JSONArray data = createResponseData(fields, resultElements);
		return new ResponseObject(ResponseStatusCode.SUCCESS, data);
	}

	private List<String> createFieldsListFromParameter(JSONArray properties) throws USMJSONAPIException {
		int length = properties.length();
		List<String> fields = new ArrayList<String>(length);
		for (int i = 0; i < length; i++) {
			try {
				fields.add(properties.getString(i));
			} catch (JSONException e) {
				throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.READ_DATA_JSON_ERROR_2, e);
			}
		}
		return fields;
	}

	private JSONArray createResponseData(List<String> fields, DataObject[] resultElements) throws USMJSONAPIException {
		JSONArray data = new JSONArray();
		if (resultElements != null) {
			for (DataObject dataObject : resultElements) {
				try {
					data.put(storeRequestedFieldsInJSONObject(dataObject, fields));
				} catch (JSONException e) {
					throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.READ_DATA_JSON_ERROR, e);
				}
			}
		}
		return data;
	}

    private DataObject[] readDataElements(long syncID, String folderID, JSONArray objectids, List<String> fields) throws USMJSONAPIException {
        try {
            Folder folder = getFolderByUUID(folderID);
			DataObjectSet cachedFolderElements = new DataObjectSet(_session.getCachedFolderElements(folder.getID(), folder
					.getElementsContentType(), syncID));
            List<DataObject> resultElements = new ArrayList<DataObject>();
            String[] fieldsArray = fields.toArray(new String[fields.size()]);
            int length = objectids.length();
            for (int i = 0; i < length; i++) {
                String objectUUID = objectids.getString(i);
                try {
                    String oxObjectID = getDataObjectByUUID(cachedFolderElements, extractUUIDFromString(objectUUID)).getID();
                    DataObject readObject = _session.readDataObject(folder.getID(), oxObjectID, fieldsArray);
                    if (readObject != null) {
                        if (readObject.getUUID() == null)
                            readObject.setUUID(java.util.UUID.fromString(objectUUID));
                        resultElements.add(readObject);
                    }
                } catch (DataObjectNotFoundException e) {
                    throw USMJSONAPIException.createObjectNotFoundException(e);
                } catch (MultipleOperationsOnDataObjectException e) {
                    throw USMJSONAPIException.createMultipleOperationsOnDataObjectException(e);
                } catch (InvalidUUIDException e) {
                    throw USMJSONAPIException.createInvalidUUIDException(e);
                }
            }
            return resultElements.toArray(new DataObject[resultElements.size()]);
        } catch (USMJSONAPIException e) {
            throw e;
        } catch (USMException e) {
            throw USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.GET_CACHED_DATA_GET_ELEMENTS_ERROR, e);
        } catch (JSONException e) {
            throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.READ_DATA_JSON_ERROR, e);
        }
    }

	protected JSONObject storeRequestedFieldsInJSONObject(DataObject o, List<String> fields) throws JSONException {
		JSONObject result = new JSONObject();
		for (String field : fields) {
			Object fieldContent = o.getFieldContent(field);
			DataType<?> fieldType = o.getContentType().getFields()[o.getFieldIndex(field)].getFieldType();
			fieldType.storeInJSONObject(o.getSession(), result, field, fieldContent);
		}
		result.put(UUID, o.getUUID().toString());
		return result;
	}

	@Override
	protected String[] getOptionalParameters() {
		return OPTIONAL_PARAMETERS;
	}

	@Override
	protected String[] getRequiredParameters() {
		return REQUIRED_PARAMETERS;
	}
}
